package ch.programmieraufgaben.planetensortieren.controller;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import ch.programmieraufgaben.planetensortieren.gui.ControllPanel;
import ch.programmieraufgaben.planetensortieren.gui.GUI;

/**
 * Rufe die vom Anwender gewählte Sortiermethode (Algorithmus) 
 * in einem eigenen Thread (Prozess) auf.
 * Der Therad kann durch den user aus dem GUI mit "stop()" gestoppt 
 * werden. Dies ruft die nicht mehr aktuelle Methode "Thread.stop()" auf.
 * Eine Möglichkeit, dies zu umgehen, wäre es, in den Sortiermethoden des
 * Users eine Exception zu werfen, und so die Sortierung abzubrechen.
 * Wer hat hier einen besseren Vorschlag?
 * 
 * @author Philipp Gressly (phi AT gressly DOT ch)
 */
public class SorterRunnable implements Runnable {

    ControllPanel controllerPanel; // welche Sortiermethode
    GUI           gui;
    
    
    public SorterRunnable(ControllPanel cp, GUI g) {
        this.controllerPanel = cp;
        this.gui = g;   }
  
    
    public void run() {
        Method sortMethod = controllerPanel.getSortMethod();
        Sortierer s = new Sortierer();
        s.setGUI(gui);
        try {
            sortMethod.invoke(s, new Object[]{}); } 
        catch (InvocationTargetException ite) {
            if(ite.getCause() instanceof ThreadDeath) {
                /* ok: user stopped */  } 
            else {
                System.out.println("error in sorter Runnable: " + ite);
                ite.printStackTrace();  }
        } catch (Exception e) {
            System.err.println("Error in SorterRunnable: " + e);
            e.printStackTrace();  } 
        controllerPanel.stoppRunningThread();  }

}  // end of class SorterThread
