package ch.programmieraufgaben.planetensortieren.controller;

import ch.programmieraufgaben.planetensortieren.gui.GUI;
import ch.programmieraufgaben.planetensortieren.model.Sonnensystem;

/**
 * Basisklasse zum Sortieren.
 * Zur Verfügung gestellt werden die folgenden Funktionen:
 *    a) getFirstSortPos()      : Erste Array Position (normalerweise 0)
 *    b) getLastSortPos()       : Letzte Array position (Bei 8 Planeten -> 7)
 *    c) tausche(pos1, pos2)    : Tausche die Planeten an Pos1 mit Pos2
 *    d) istKleiner(pos1, pos2) : gibt true (WAHR) zurück, genau dann, wenn der Planet
 *                                an position "pos1" kleiner ist als derjenige an "pos2".
 *                                ACHTUNG: bei Gleichheit gibt die Methode "false" aus!
 *                                "kleiner" bezieht sich auf das vom Anwender gewählte Kriterium
 *                                z. B. "Name", "Abstand zur Sonne", ...  
 *    e) istSortiert()          : ein tiviales Vorgehen, um zu testen, ob die Daten sortiert sind.
 *                                Von links nach rechts wird paarweise verglichen.
 *                                Verwende diesen Algorithmus sparsam, denn in der Regel gibt es
 *                                raschere Algorithmen.
 *
 * @author Philipp Gressly (phi AT gressly DOT ch)
 */
public class SortierBasis {
    
    Sonnensystem sonnensystem;
    GUI gui;
    
       
    public void setGUI(GUI gui) {
        this.gui = gui;
        this.sonnensystem  = gui.getSonnensystem();  }
    
    
    int getFirstSortPos() {
        return 0;  } // Arrays in Java starten immer mit Index 0
    
    
    int getLastSortPos() { 
        return sonnensystem.size() - 1; }
    
    
    void tausche(int pos1, int pos2) {
      if(pos2 > pos1) {
          int tmp = pos1;
             pos1 = pos2;
             pos2 = tmp;  }
      sonnensystem.tausche(pos1, pos2);  }
    
    
    boolean istKleiner(int pos1, int pos2) {
       return sonnensystem.istKleiner(pos1, pos2);  }
    
    
    boolean istSortiert() {
        int p = getFirstSortPos();
        while(p <= getLastSortPos() - 1) {
            if(!this.istKleiner(p, p + 1)) {
                return false;   }
            p = p + 1;  }
        return true;  }
    
}
 // end of class SortierBasis
