package ch.programmieraufgaben.planetensortieren.gui;

import static eu.gressly.hw.iteration.Sequencer.seq;

/**
 * Erzeuge Bezier-Interpolationspunkte für Bezierkurven mit zwei Zwischenpunkten (Grad 3).
 * @author Philipp Gressly (phi@gressly.ch)
 */
/*
 * History: first Implementation: Jul 18, 2009
 * Bugs   :
 */
public class BezierGrad3 {

    static double [][] bezFactors     ;
    double []          values         ;
    static int         INTERSECTS = 64; // == 64 Steps = 65 Points
    
    
    public BezierGrad3(int x0, int x1, int x2, int x3){
       if(null == bezFactors) {
           initParams();  }
       values = new double[INTERSECTS + 1];
       calcValues(x0, x1, x2, x3);  }
    
    
    private void calcValues(int x0, int x1, int x2, int x3) {
        for(int i : seq(0, INTERSECTS)) { 
             this.values [i] = bezFactors[i][0] * x0 + 
                               bezFactors[i][1] * x1 +
                               bezFactors[i][2] * x2 +
                               bezFactors[i][3] * x3; }
    }
    
    // 3. Zeile im Pascalschen Dreieck:
    int kOverN3[] = new int [] {1, 3 , 3, 1}; 
     
    /** 
     * Das vorberechnen der parameter spart einiges an Rechenzeit.
     * Danke Martin Blatter für den Tipp.
     */
    private void initParams() {
       double divisor;
       divisor = Math.pow((double)INTERSECTS, 3.0);
       bezFactors = new double[INTERSECTS+1][4];
       for(int k: seq(0, 3)) {
           int kn = kOverN3[k];
           for(int i : seq(0, INTERSECTS)){ 
               bezFactors[i][k] = (double) 
                                    ((double) kn * 
                                     Math.pow ((double) i, (double) k) * 
                     Math.pow ((double) (INTERSECTS-i), (double) (3-k)) / divisor); }
       }    
    }

    
    public int get(int i) {
        return (int) values[i];  }

}  // end of class Bezier2d
