package ch.programmieraufgaben.planetensortieren.gui;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.event.*;

import ch.programmieraufgaben.planetensortieren.controller.SorterRunnable;
import ch.programmieraufgaben.planetensortieren.controller.Sortierer;
import ch.programmieraufgaben.planetensortieren.controller.Sort;
import ch.programmieraufgaben.planetensortieren.model.Planet;

/**
 * 
 * Das "Controll-Panel" steuert, wie und was sortiert werden soll:
 *   - Start, Stopp und Mischen
 *   - Algorithmus und visuelle Geschwindigkeit
 *   - Sortierkriterium und ob aufsteigend oder absteigend.
 * @author Philipp Gressly (phi@gressly.ch)
 *//*
 * History: first Implementation: Jul 23, 2009
 * Bugs   :
 */

public class ControllPanel extends JPanel implements ActionListener,
    ItemListener, ChangeListener {
    JButton      start, stopp, misch;
    JComboBox    sortAlgoBox, sortByBox;
    JRadioButton ascButton, descButton;
    JSlider      vitesseSlider;
    private GUI  gui;

    
    public ControllPanel(GUI gui){
        this.gui = gui;
        fillSortByBox();
        fillAlgoBox();
        generateAndLayout();  }

    
    private void fillAlgoBox() {
        sortAlgoBox = new JComboBox();
        Class<Sortierer> sortiererClass = Sortierer.class;
        Method[] methods = sortiererClass.getDeclaredMethods();
        for (Method m : methods) {

           
            if (m.isAnnotationPresent(Sort.class)) {
                sortAlgoBox.addItem(m.getName());    }
        }
    }

    
    /**
     * Füge alle Attribute der Klasse "Planet" in die Combobox.
     */
    private void fillSortByBox() {
        sortByBox = new JComboBox();
        Class<Planet> planetClass = Planet.class;
        Field fields[] = planetClass.getFields();
        for (Field f : fields) {
            String kriterium = f.getName();
            if(! "id".equals(kriterium)) {
            sortByBox.addItem(kriterium);}  }
    }

    ResourceBundle rb = ResourceBundle.getBundle("ch.programmieraufgaben.planetensortieren.gui.text");
    
    private void generateAndLayout() {
        setLayout(new GridBagLayout());
        
        layoutButtonColumn();
        layoutSortColumn();
        layoutCriteriaColumn();
        
        addListeners();  }

    
    /**
     * Start, Stopp und Mischen Buttons
     */
    private void layoutButtonColumn() {
        start = new JButton(rb.getString("label.start"));
        addGB(start, 1, 1);
        stopp = new JButton(rb.getString("label.stopp"));
        stopp.setEnabled(false);
        addGB(stopp, 1, 2);
        misch = new JButton(rb.getString("label.mix"));
        addGB(misch, 1, 3);    }

    
    /**
     * Algorithmus (Dropdown) & Geschwindigkeit (vitesse-slider)
     */
    private void layoutSortColumn() {
        addGB(new JLabel(rb.getString("label.algo") + ":"), 2, 1);
        addGB(sortAlgoBox, 2, 2);
        addGB(new JLabel(rb.getString("label.vitesse") + ":"), 2, 3);
        vitesseSlider = new JSlider(10, 2000);
        vitesseSlider.setName("km/h");
        addGB(vitesseSlider, 2, 4);   }
    
    
    /**
     * Kriterium-Box, Aufsteigend/Absteigend (Radio-Buttons)
     */
    private void layoutCriteriaColumn() {
        addGB(new JLabel(rb.getString("label.sort.krit") + ":"), 3, 1);
        addGB(sortByBox, 3, 2);
        sortByBox.setRenderer(new SortByDropdownRenderer());
        ascButton = new JRadioButton(rb.getString("button.asc"));
        descButton = new JRadioButton(rb.getString("button.desc"));
        ButtonGroup bg = new ButtonGroup();
        bg.add(ascButton);
        bg.add(descButton);
        ascButton.setSelected(true);
        addGB(ascButton, 3, 3);
        addGB(descButton, 3, 4);  }

    
    private void addListeners() {
        start.addActionListener(this);
        stopp.addActionListener(this);
        misch.addActionListener(this);
        sortAlgoBox.addItemListener(this);
        sortByBox.addItemListener(this);
        ascButton.addActionListener(this);
        descButton.addActionListener(this);
        vitesseSlider.addChangeListener(this);  }

    
    /**
     * In ein GridBag-Layout anordnen.
     */
    GridBagConstraints gbc = new GridBagConstraints();

    private void addGB(JComponent controll, int x, int y) {
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.insets = new Insets(3, 3, 3, 3);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.anchor = GridBagConstraints.WEST;
        add(controll, gbc);  }

    
    public static void main(String[] args) {
        JFrame jf = new JFrame("Test ControllPanel");
        ControllPanel cp = new ControllPanel(null);
        jf.add(cp);
        jf.pack();
        jf.setVisible(true);  }

    public String getKriterium() {
        String selected = (String) sortByBox.getSelectedItem();
        return selected;  }

    /*
     * @see
     * java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    Thread st;
    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == start) {
            startAction();
            return; }
        if(e.getSource() == stopp) {
          st.interrupt();
          return;    }
        if(e.getSource() == misch) {
            gui.getSonnensystem().mische();
            gui.validateSonnensystemPanel();
            return;  }
    }
    
   
    /**
     * Sortirprozess starten.
     */
    private void startAction() {
        disableButtons();
        Runnable sr = new SorterRunnable(this, gui);
        st = new Thread(sr);
        st.start();  }

    
    /**
     * Sobald der Sortierprozess beendet ist oder der User diesen Prozess
     * unterbrochen hat, sollen die Buttons wieder aktiviert (enabled) werden.
     */
    public void stoppRunningThread() {
        enableButtons();
        validate();  }
    
    
    /**
     * Nur noch stopp und die geschwindigkeit sind zu haben
     */
    private void disableButtons() {
        changeButtonsStates(true);    }
    private void enableButtons() {
        changeButtonsStates(false);    }
    
    private void changeButtonsStates(boolean disable) {
        start.setEnabled(!disable);
        stopp.setEnabled(disable);
        misch.setEnabled(!disable);
        sortByBox.setEnabled(!disable);
        ascButton.setEnabled(!disable);
        descButton.setEnabled(!disable);
        sortAlgoBox.setEnabled(!disable);   }
    
    
    /*
     * @see
     * java.awt.event.ItemListener#itemStateChanged(java.awt.event.ItemEvent)
     */
    @Override
    public void itemStateChanged(ItemEvent e) {
        if (e.getSource() == sortByBox) {
            gui.setKriterium((String) e.getItem());
        }
    }

    
    /*
     * @see
     * javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent
     * )
     */
    @Override
    public void stateChanged(ChangeEvent e) { }

    public Method getSortMethod() {
        Class<Sortierer> sortiererClass = Sortierer.class;
        String methodName = (String) sortAlgoBox.getSelectedItem();
        Method m = null;
        try {
            m = sortiererClass.getDeclaredMethod(methodName);
        } catch (Exception e) {
            System.err.println("Error in ControllPanel.getSortMethod: " + e);
            e.printStackTrace();
        }
        return m;   }

    
    /**
     * Wie schnell soll sortiert werden? 
     * Lese den "JSlider" aus.
     */
    public long getVitesse() {
        return 2010 - vitesseSlider.getValue();   }


    public boolean isAscending() {
        return ascButton.isSelected();   }

    
} // end of class ControllPanel
