package ch.programmieraufgaben.planetensortieren.model;

import java.lang.reflect.Field;

import ch.programmieraufgaben.planetensortieren.gui.GUI;


/**
 * Jeder Himmelskörper hat eine id (z. B. "terra") und einen 
 * sprachabhängigen Namen (z. B. "Erde" oder "Earth").
 * Der Vergleich "istKleinerAls" vergleicht die Attribute in
 * "natürlicher" weise : Strings alphabetisch und "double" nach Wert.
 * 
 * @author Philipp Gressly (phi AT gressly DOT ch)
 */
public class Himmelskoerper {
   GUI gui;
   /**
    * "img/" + name + ".png" = image-file    
    */
    public String id; // sprachunabhängig
    public String name; // sprachbezogen
    
    
    public Himmelskoerper(String id, GUI gui) {
        this.id   = id;
        this.gui  = gui;
    }
    
    
    /**
     * Vergleicht das aktuelle (vom Anwender gewählte) Kriterium
     * entweder alphabetisch (String) oder nach Zahlwert (double).
     * (Andere Datentypen sind noch nicht erfasst.)
     * @param anderer ist der 2. Himmelskörper, mit dem verglichen werden soll.
     * @return true, genau dann, wenn dieser (this) Himmelskörper (echt) kleiner als der andere (anderer) ist.
     */
    public boolean istKleinerAls(Himmelskoerper anderer) {
       try{ 
         String kriterium = gui.getKriterium(); 
         Class<Planet> plClass = Planet.class;
         Field f = plClass.getField(kriterium);
         if(f.getType().equals(String.class)) {
           String s1 = (String) f.get(this);
           String s2 = (String) f.get(anderer);
           return s1.compareTo(s2) < 0;  }
         else {
           double val1 = f.getDouble(this);
           double val2 = f.getDouble(anderer);
           return val1 < val2; }
       } catch (Exception ex) {
           System.err.println("Fehler in Himmelskoerper.istKleinerAls() : " + ex);
           ex.printStackTrace(); }
       return false;  }
    
    
    public String toString() {
        return "Himmelskoerper: " + id + "(" + name + ")";  }

}  // end of class Himmelskoerper