package eu.gressly.hw.iteration;

import java.util.Iterator;

/**
 * A simple to use sequecer for the new java "foreach" loop.
 * Use
 * <code>for(int x : seq(a, b))</code>
 * instead of
 * <code>for(int x = a; x <= b; x = x + 1)</code>.
 * 
 * Example to produce all domino stones: 
 * <pre>
 * import static eu.gressly.hw.iteration.Sequencer.seq;
 * 
 * for(int obereZahl : seq(0, 6)) {
 *          for(int untereZahl : seq(obereZahl, 6)) {
 *              System.out.println("("+ obereZahl + "|" + untereZahl+ ")");
 *          }
 *      }</pre>
 *  
 * Or simply use a code 100 times:
 * <pre>
 *   for(int i : seq(100)) { // i is not used
 *   myCode(); }
 * </pre>
 *      
 *      
 * 
 * @author Philipp Gressly (phi@gressly.ch)
 *//*
 * History: first Implementation: May 26, 2009
 * Bugs   : not yet known any
 */


public class Sequencer {
    
    class FromToIterator  implements Iterator<Integer> {
        int from, to, act;
        FromToIterator(int from, int to) {
            this.from = from;
            this.to = to;
            act = from;
        }
        
        FromToIterator(int to) {
            this(1, to);
        }
    
        @Override
        public boolean hasNext() {
           return act <= to;
        }
    
        @Override
        public Integer next() {
           return new Integer(act++);
        }
      
        @Override
        public void remove() {
           act ++;
        }
    } // end class FromToIterator
    
    class FromToIterable implements Iterable<Integer> {
        FromToIterator fti;
        FromToIterable(int from, int to) {
            this.fti  = new FromToIterator(from, to);
        }
        FromToIterable(int to) {
            this(1, to);
        }
        public Iterator<Integer> iterator() {
            return this.fti;
        }
        
    } // end class FromToIterable
    
   public static Iterable<Integer> seq(int from, int to) {
           return new Sequencer().new  FromToIterable(from, to);
   }
   public static Iterable<Integer> seq(int to) {
       return Sequencer.seq(1, to);
   }
   
}  // end of class Sequencer